#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
from __future__ import division
"""
A backend that can be used to test Rayon code programmatically.
"""
from rayon import RayonException

import backend



class Context(object):
    def __init__(self, width, height):
        self._width  = width
        self._height = height
    def canvas_dimensions(self):
        return (self._width, self._height)
    def draw_bitmap(self, bitmap, x, y, width, height, fill):
        return
    def draw_image(self, filename, x, y, width, height, fill):
        return
    def draw_ellipse(self, x, y, width, height, stroke_color,
                     fill_color, stroke_width=1):
        return
    def draw_line(self, x1, y1, x2, y2, color, width, fixup=False,
                  dashes=None, dash_offset=0):
        return
    def draw_lines(self, segments, color, width, fixup=False,
                   dashes=None, dash_offset=0):
        return
    def draw_polygon(self, vertices, stroke_width, stroke_color, fill_color,
                     fixup=False):
        return
    def draw_rectangle(self, x, y, width, height,
                       stroke_width, stroke_color, fill_color,
                       fixup=False):
        return
    def draw_text(self, text, bbox, size, family, style, weight,
                  color=(0, 0, 0), angle=None):
        return
    def get_text_dimensions(self, text, size, family, style, weight):
        return (0, 0)
    def push_state(self):
        return
    def pop_state(self):
        return
    def translate(self, dx, dy):
        return
        

class DebugContext(object):
    # Proxy of RealContext object that prints the method being called
    # out to the console.
    def __init__(self, logger, width, height):
        self._logger = logger
        self._real = Context(width, height)
    def __getattr__(self, name):
        if hasattr(self.real, name):
            attr = getattr(self.real, name)
            def wrapper(*args, **kwargs):
                self._logger(name, args, kwargs)
                return attr(*args, **kwargs)
            return wrapper
        else:
            raise AttributeError(name)




class Page(backend.Page):
    @classmethod
    def from_logger(cls, logger, width, height):
        cls(logger, width, height)
    def __init__(self, logger, width, height):
        self._ctx = DebugContext(logger, width, height)
    def write(self, chart):
        chart.layout(self._ctx)
        
