#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
# Linear-time selection algorithm (quickselect). Used to get medians
# for kd-tree. Cribbed from
# http://en.wikipedia.org/wiki/Selection_algorithm (Suspiciously
# absent section: "Selection Algorithms in Popular Culture")

#import copy
import random



def partition(x, left, right, pivot_idx, key):
    # Do a quicksort-style partition of the data.
    pivot_val = x[pivot_idx]
    x[pivot_idx], x[right] = x[right], x[pivot_idx]
    store_idx = left
    for i in xrange(left, right):
        if key(x[i]) < key(pivot_val):
            x[store_idx], x[i] = x[i], x[store_idx]
            store_idx += 1
    x[right], x[store_idx] = x[store_idx], x[right]
    return store_idx, x


def select(x, k, left, right, key=lambda _: _):
    """Return the k-th element of unsorted list x, if x were sorted
    using key(). Generally takes linear time, which is usually
    better than running a sort of x. (If len(x) <= 5, we just sort and
    take the kth element.)

    x will be sorted in place such that all x[:k] <= x[k] and x[k:] >= x[k]"""
    if len(x) <= 5:
        x.sort(key=key)
        return x[k]
    while True:
        pivot_idx = random.randint(left, right)
        new_pivot_idx, x = partition(x, left, right, pivot_idx, key)
        if k == new_pivot_idx:
            return x[k]
        elif k < new_pivot_idx:
            right = new_pivot_idx - 1
        else:
            left = new_pivot_idx + 1




######
