#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@

from __future__ import division

import unittest


from rayon.geometry import Rectangle, ScreenPoint, Spacing


def to_points(*t_pts):
    return tuple(ScreenPoint(*t) for t in t_pts)

class t_Spacing(unittest.TestCase):
    def test_access_1(self):
        spc = Spacing(1,2,3,4)
        tpad, bpad, lpad, rpad = spc
        self.assertEqual((tpad, bpad, lpad, rpad), (1,2,3,4))
    def test_defaults_1(self):
        spc = Spacing(1)
        tpad, bpad, lpad, rpad = spc
        self.assertEqual((tpad, bpad, lpad, rpad), (1,0,0,0))
    def test_named_params_1(self):
        spc = Spacing(tpad=1)
        tpad, bpad, lpad, rpad = spc
        self.assertEqual((tpad, bpad, lpad, rpad), (1,0,0,0))
    def test_named_params_2(self):
        spc = Spacing(bpad=1)
        tpad, bpad, lpad, rpad = spc
        self.assertEqual((tpad, bpad, lpad, rpad), (0,1,0,0))
    def test_named_params_3(self):
        spc = Spacing(lpad=1)
        tpad, bpad, lpad, rpad = spc
        self.assertEqual((tpad, bpad, lpad, rpad), (0,0,1,0))
    def test_named_params_4(self):
        spc = Spacing(rpad=1)
        tpad, bpad, lpad, rpad = spc
        self.assertEqual((tpad, bpad, lpad, rpad), (0,0,0,1))

class t_Rectangle(unittest.TestCase):
    r1 = Rectangle(100, 50)
    def test_corners_1(self):
        self.assertEqual(self.r1.get_corners(),
                         to_points((0,0), (100,0),
                                   (0, 50), (100,50)))
    def test_corners_2(self):
        self.assertEqual(self.r1.get_corners(top_left=ScreenPoint(10,20)),
                         to_points((10,20), (110,20),
                                   (10, 70), (110,70)))
    def test_corners_3(self):
        self.assertEqual(self.r1.get_corners(top_right=ScreenPoint(10,20)),
                         to_points((-90,20), (10,20),
                                   (-90, 70), (10,70)))
    def test_corners_4(self):
        self.assertEqual(self.r1.get_corners(bottom_left=ScreenPoint(10,20)),
                         to_points((10,-30), (110,-30),
                          (10, 20), (110,20)))
    def test_corners_5(self):
        self.assertEqual(self.r1.get_corners(bottom_right=ScreenPoint(10,20)),
                         to_points((-90,-30), (10,-30),
                                   (-90, 20), (10,20)))
    def test_corners_6(self):
        self.assertEqual(self.r1.get_corners(center=ScreenPoint(10,20)),
                         to_points((-40,-5), (60,-5),
                                   (-40, 45), (60,45)))
    def test_width_1(self):
        self.assertEqual(self.r1.get_width(), 100)
    def test_height_1(self):
        self.assertEqual(self.r1.get_height(), 50)
    def test_eq_1(self):
        self.assertEqual(self.r1, Rectangle(100, 50))
    def test_neq_1(self):
        self.assertNotEqual(self.r1, Rectangle(99, 50))
    def test_neq_2(self):
        self.assertNotEqual(self.r1, Rectangle(100, 49))
    def test_new_3(self):
        self.assertNotEqual(self.r1, (100, 50))
    def test_pad_1(self):
        self.assertEqual(self.r1.pad(Spacing(1, 2, 3, 4)),
                         (Rectangle(93, 47), ScreenPoint(3, 1)))
    def test_pad_2(self):
        self.assertEqual(self.r1.pad(Spacing(0,0,0,0)),
                         (self.r1, ScreenPoint(0, 0)))
    def test_pad_3(self):
        # Negative padding is permitted. If you want to leave your
        # clipping box, it's your business.
        self.assertEqual(self.r1.pad(Spacing(-1,-2,-3,-4)),
                         (Rectangle(107, 53), ScreenPoint(-3, -1)))
    def test_pad_5(self):
        # Padding away the rectangle is permitted.
        self.assertEqual(self.r1.pad(Spacing(25, 25, 50, 50)),
                         (Rectangle(0, 0), ScreenPoint(50, 25)))
    def test_pad_6(self):
        # More padding than rectangle area results in a 0x0 rect,
        # whose top left corner depends on whatever ludicrous values
        # were given to tpad and lpad.
        self.assertEqual(
            self.r1.pad(Spacing(1000, 2000, 3000, 4000)),
            (Rectangle(0, 0), ScreenPoint(3000, 1000)))
    def test_pad_err_1(self):
        self.assertRaises(TypeError, lambda: self.r1.pad(tuple()))
            
        
        
    
        





        
if __name__ == '__main__':
    unittest.main()
