#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
import unittest
import sys
from pprint import pprint as pprint
from StringIO import StringIO

from rayon import rytools
from rayon.rytools import *
# Parts of the private interface we still want to test:
from rayon.rytools import (parse_command_line,
                           parse_config_file,
                           merge_configuration,
                           ConverterOption,
                           NO_DEFAULT)

from netsa.data.times import tzinfo_UTC

from datetime import datetime, timedelta

def mkmain(mainfunc):
    exc = []
    if mainfunc is None:
        return None, exc
    def main(opts):
        try:
            return mainfunc(opts)
        except Exception, e:
            exc.append(e)
            return 500
    return main, exc

class t_OptionParsing(unittest.TestCase):
    def run_execute(self,
                    cmd_name=None,
                    cmd_func=None,
                    cmd_args=None,
                    cmd_options=None,
                    include_common_options=True,
                    cmd_desc=None):
        newmain, exc = mkmain(cmd_func)
        try:
            rytools.execute(cmd_name=cmd_name,
                            cmd_func=newmain,
                            cmd_args=cmd_args,
                            cmd_options=cmd_options,
                            include_common_options=include_common_options,
                            cmd_desc=cmd_desc,
                            debug=True)
        except SystemExit, e:
            if e.code != 0:
                if len(exc):
                    raise exc[0]
                else:
                    self.fail("Execute exited unexpectedly with error code %s" %
                              e.code)
                

    def test_extra_args_1(self):
        msg = [""]
        def main(opts):
            self.assertEqual(opts['__EXTRA__'], ('a', 'b', 'c'))
        test_args = "--output-path=foo a b c".split(" ")
        self.run_execute(cmd_func=main, cmd_args=test_args)


    # This function also has tricky logic related to which
    # configuration files it parses and when. At the moment, this
    # should be done manually; bin/rytestapp contains a rudimentary
    # test plan that should check things.

class t_Option(unittest.TestCase):
    def test_access_1(self):
        optname, valname, default_val, hlp, metavar = Option("foo")

    def test_optname_1(self):
        self.assertEqual(Option("--foo")[0], "foo")
    def test_optname_2(self):
        self.assertEqual(Option("foo-bar")[0], "foo-bar")
        
    def test_valname_1(self):
        self.assertEqual(Option("foo")[1], "foo")
    def test_valname_2(self):
        self.assertEqual(Option("--foo")[1], "foo")
    def test_valname_3(self):
        self.assertEqual(Option("foo-bar")[1], "foo_bar")
    def test_valname_3(self):
        self.assertEqual(Option("foo", "bar")[1], "bar")

    def test_default_val_1(self):
        self.assertEqual(Option("foo")[2], NO_DEFAULT)
    def test_default_val_2(self):
        self.assertEqual(Option("foo", "foo", "bar")[2], "bar")
    def test_default_val_3(self):
        self.assertEqual(Option("foo", default_val="bar")[2], "bar")

    def test_help_1(self):
        self.assertEqual(Option("foo")[3], SUPPRESS_HELP)
    def test_help_2(self):
        self.assertEqual(Option("foo", "bar", "baz", "quux")[3], "quux")
    def test_help_3(self):
        self.assertEqual(Option("foo", hlp="quux")[3], "quux")

    # The converter converts the option input (a string) into the
    # appropriate type of thing. By default, it's the identity
    # function
    def test_converter_1(self):
        self.assertEqual(Option("foo")("bar"), "bar")
    def test_converter_2(self):
        self.assertEqual(Option("foo", "bar", "baz", "quux", int)("10"),
                         10)
    def test_converter_3(self):
        self.assertEqual(Option("foo", converter=int)("10"), 10)

# UnaryOption objects are empty subclasses of Option, the tests
# above for Option suffice.

class t_ListOption(unittest.TestCase):
    def test_defaults_1(self):
        self.assertEqual(tuple(ListOption("foo")),
                         ("foo",         # name
                          "foo",         # variable name
                          NO_DEFAULT,    # default value
                          SUPPRESS_HELP, # help text
                          "ARGS",        # metavariable
                          ","))          # list delimiter
    def test_delim_1(self):
        self.assertEqual(ListOption("foo")("1,2,3"), ('1', '2', '3'))
    def test_delim_2(self):
        self.assertEqual(ListOption("foo")("1, 2, 3"), ('1', '2', '3'))
    def test_delim_3(self):
        self.assertEqual(ListOption("foo")(""), tuple())
    def test_delim_4(self):
        self.assertEqual(ListOption("foo")("1"), ("1",))
    def test_custom_delim_1(self):
        self.assertEqual(ListOption("foo", delimiter=";")("1;2;3"),
                         ('1', '2', '3'))
    def test_converter_1(self):
        self.assertEqual(ListOption("foo", converter=int)("1,2,3"), (1,2,3))

class t_ChoiceOption(unittest.TestCase):
    def test_defaults_1(self):
        self.assertEqual(tuple(ChoiceOption("foo", choices=('a', 'b'))),
                         ("foo",         # name
                          "foo",         # variable name
                          NO_DEFAULT,    # default value
                          SUPPRESS_HELP, # help text
                          "ARGS",        # metavariable
                          ('a', 'b')))   # choices
    def test_choices_1(self):
        self.assertEqual(ChoiceOption("foo", choices=('a', 'b'))('a'), 'a')
    def test_choices_2(self):
        self.assertRaises(
                ToolRuntimeError,
                lambda: ChoiceOption("foo", choices=('a', 'b'))('c'))
    def test_converter_1(self):
        self.assertEqual(
            ChoiceOption("foo", choices=(1,2), converter=int)('1'), 1)
    def test_converter_2(self):
        self.assertRaises(
            ValueError,
            lambda: ChoiceOption("foo", choices=(1,2), converter=int)("a"))
                        
            
         

class t_KVOption(unittest.TestCase):
    def test_defaults_1(self):
        self.assertEqual(tuple(KVOption("foo")),
                         ("foo",         # name
                          "foo",         # variable name
                          NO_DEFAULT,    # default value
                          SUPPRESS_HELP, # help text
                          "ARG",         # metavariable
                          ",",           # list delimiter
                          "="))          # pair delimiter
    def test_delim_1(self):
        self.assertEqual(KVOption("foo")("a=1,b=2"), (('a', '1'), ('b', '2')))
    def test_delim_2(self):
        self.assertEqual(KVOption("foo")("a = 1, b = 2"),
                         (('a', '1'), ('b', '2')))
    def test_delim_3(self):
        self.assertEqual(KVOption("foo")(""), tuple())
    def test_delim_4(self):
        self.assertEqual(KVOption("foo")("a=1"), (('a', '1'),))
    def test_delim_5(self):
        self.assertEqual(KVOption("foo")("a="), (('a', ''),))
    def test_delim_6(self):
        self.assertEqual(KVOption("foo")("a=,b=2"), (('a', ''), ('b', '2')))
    def test_custom_delim_1(self):
        self.assertEqual(KVOption("foo",
                                  delimiter=";",
                                  kv_delimiter=",")("a,1;b,2"),
                         (('a', '1'), ('b', '2')))
    def test_converter_1(self):
        self.assertEqual(KVOption("foo", converter=int)("a=1,b=2"),
                         (('a', 1), ('b', 2)))
    def test_err_1(self):
        o = KVOption("foo")
        self.assertRaises(ValueError, lambda: o("a"))
    def test_err_2(self):
        o = KVOption("foo")
        self.assertRaises(ValueError, lambda: o("a,b"))
    def test_err_2(self):
        o = KVOption("foo")
        self.assertRaises(ValueError, lambda: o("="))
    def test_err_2(self):
        o = KVOption("foo")
        self.assertRaises(ValueError, lambda: o("=1"))
    def test_err_2(self):
        o = KVOption("foo")
        self.assertRaises(ValueError, lambda: o("a=1,b"))
    def test_err_2(self):
        o = KVOption("foo")
        self.assertRaises(ValueError, lambda: o("a=1,,b=2"))


class t_Flag(unittest.TestCase):
    def test_defaults_1(self):
        self.assertEqual(tuple(Flag("foo")),
                         ("foo",
                          "foo",
                          False,
                          SUPPRESS_HELP,
                          "FLAG"))

    def test_optname_1(self):
        self.assertEqual(Flag("--foo")[0], "foo")
    def test_optname_2(self):
        self.assertEqual(Flag("foo-bar")[0], "foo-bar")
        
    def test_valname_1(self):
        self.assertEqual(Flag("foo")[1], "foo")
    def test_valname_2(self):
        self.assertEqual(Flag("--foo")[1], "foo")
    def test_valname_3(self):
        self.assertEqual(Flag("foo-bar")[1], "foo_bar")
    def test_valname_3(self):
        self.assertEqual(Flag("foo", "bar")[1], "bar")

    def test_default_val_1(self):
        self.assertEqual(Flag("foo")[2], False)
    def test_default_val_2(self):
        self.assertRaises(TypeError,
                          lambda: Flag("foo", default_val=9))

    def test_help_1(self):
        self.assertEqual(Flag("foo")[3], SUPPRESS_HELP)
    def test_help_2(self):
        self.assertEqual(Flag("foo", "bar", "quux")[3], "quux")
    def test_help_3(self):
        self.assertEqual(Flag("foo", hlp="quux")[3], "quux")
    

    def test_converter_1(self):
        self.assertEqual(Flag("foo")(0), False)
    def test_converter_2(self):
        self.assertEqual(Flag("foo")(False), False)
    def test_converter_3(self):
        self.assertEqual(Flag("foo")("0"), False)
    def test_converter_4(self):
        self.assertEqual(Flag("foo")("False"), False)
    def test_converter_5(self):
        self.assertEqual(Flag("foo")("false"), False)
    def test_converter_6(self):
        self.assertEqual(Flag("foo")("FaLsE"), False)
    def test_converter_7(self):
        self.assertEqual(Flag("foo")("Anything else"), True)

        
class t_FalseFlag(unittest.TestCase):
    def test_defaults_1(self):
        self.assertEqual(tuple(FalseFlag("foo")),
                         ("foo",
                          "foo",
                          True,
                          SUPPRESS_HELP,
                          "FLAG"))

    def test_optname_1(self):
        self.assertEqual(FalseFlag("--foo")[0], "foo")
    def test_optname_2(self):
        self.assertEqual(FalseFlag("foo-bar")[0], "foo-bar")
        
    def test_valname_1(self):
        self.assertEqual(FalseFlag("foo")[1], "foo")
    def test_valname_2(self):
        self.assertEqual(FalseFlag("--foo")[1], "foo")
    def test_valname_3(self):
        self.assertEqual(FalseFlag("foo-bar")[1], "foo_bar")
    def test_valname_3(self):
        self.assertEqual(FalseFlag("foo", "bar")[1], "bar")

    def test_default_val_1(self):
        self.assertEqual(FalseFlag("foo")[2], True)
    def test_default_val_2(self):
        self.assertRaises(TypeError,
                          lambda: FalseFlag("foo", default_val=9))

    def test_help_1(self):
        self.assertEqual(FalseFlag("foo")[3], SUPPRESS_HELP)
    def test_help_2(self):
        self.assertEqual(FalseFlag("foo", "bar", "quux")[3], "quux")
    def test_help_3(self):
        self.assertEqual(FalseFlag("foo", hlp="quux")[3], "quux")

    # For a FalseFlag, all false values are converted to True and vice
    # versa.

    def test_converter_1(self):
        self.assertEqual(FalseFlag("foo")(0), True)
    def test_converter_2(self):
        self.assertEqual(FalseFlag("foo")(False), True)
    def test_converter_3(self):
        self.assertEqual(FalseFlag("foo")("0"), True)
    def test_converter_4(self):
        self.assertEqual(FalseFlag("foo")("False"), True)
    def test_converter_5(self):
        self.assertEqual(FalseFlag("foo")("false"), True)
    def test_converter_6(self):
        self.assertEqual(FalseFlag("foo")("FaLsE"), True)
    def test_converter_7(self):
        self.assertEqual(FalseFlag("foo")("Anything else"), False)
        




# Tests for the single-valued option subclasses. We only need to test
# the converters for these; everything else is tested above.

def opt_test(oclass, metavar, inval, outval):
    def t(self):
        o = oclass("foo")
        self.assertEqual(o.get_metavariable(), metavar)
        self.assertEqual(o(inval), outval)
    return t


class t_ConverterOption(unittest.TestCase):
    class _IntOpt(ConverterOption):
        "Home-cooked IntOption for testing"
        metavar_="tester"
        converter_=int

    def test_1(self):
        o = self._IntOpt("foo")
        self.assertEqual(
            tuple(o),
            ("foo", "foo", NO_DEFAULT, SUPPRESS_HELP, "tester"))
    def test_2(self):
        o = self._IntOpt(
            "foo", valname="foo_val", default_val="foo_default", hlp="foo help")
        self.assertEqual(
            tuple(o),
            ("foo", "foo_val", "foo_default", "foo help", "tester"))
                         
        
                

class t_typed_options(unittest.TestCase):
    
    #
    # The easy ones:
    #
    test_StringOption_1 = opt_test(StringOption, "STRING", "foo"  , "foo")
    test_PathOption_1   = opt_test(PathOption,   "PATH"  , "a/b/c", "a/b/c")
    test_IntOption_1    = opt_test(IntOption,    "INT"   , "12"   , 12)

    test_ColnameOption_1 = opt_test(ColnameOption, "COLNAME_OR_IDX", "0", 0)
    test_ColnameOption_2 = opt_test(ColnameOption, "COLNAME_OR_IDX", "a", "a")
    
    test_ColorOption_1 = opt_test(
        ColorOption, "COLOR", "ff00ff00" , (255, 0, 255, 0))
    test_ColorOption_2 = opt_test(
        ColorOption, "COLOR", "#ff00ff00", (255, 0, 255, 0))
    test_ColorOption_3 = opt_test(
        ColorOption, "COLOR", "red"      , (255, 0, 0, 255))
    def test_ColorOption_4(self):
        self.assertRaises(ValueError, lambda: ColorOption("foo")("wrong"))

    test_IpOption_1 = opt_test(IpOption, "INT_OR_IP", "0.1.2.3", 0x00010203)
    test_IpOption_2 = opt_test(IpOption, "INT_OR_IP", "66051"  , 0x00010203)
    def test_IpOption_3(self):
        self.assertRaises(ValueError, lambda: IpOption("foo")("blah"))

    test_DateOption_1 = opt_test(
        DateOption, 
        "DATE", 
        "2000/01/01:00:00:00", 
        datetime(2000, 01, 01, tzinfo=tzinfo_UTC()))

    test_TimeDeltaOption_1 = opt_test(
        TimeDeltaOption, "TIMEDELTA", "PT1H", timedelta(hours=1))

    #
    # The hard ones:
    #
    def test_FloatOption_1(self):
        o = FloatOption("foo")
        self.assertEqual(o.get_metavariable(), "FLOAT")
        self.assertAlmostEqual(o("1.2"), 1.2)

    # Note that TickspecOption is deprecated in favor of
    # NewTickspecOption.
    def test_TickspecOption_1(self):
        o = TickspecOption("foo")
        self.assertEqual(o.get_metavariable(), "TICKSPEC")
        class _ColClass(object):
            def min(self): return 0
            def max(self): return 100
        c = _ColClass()
        factories = o("min,max")
        self.assertEqual(tuple(f(c) for f in factories), ([0], [100]))

    def test_NewTickspecOption_1(self):
        o = NewTickspecOption("foo")
        self.assertEqual(o.get_metavariable(), "TICKSPEC")
        class _ColClass(object):
            def percentile(self, n):
                if n == 10: return 10
                elif n == 90: return 90
                else: raise AssertionError("Unexpected call")
        c = _ColClass()
        factories = o("p(10),p(90)")
        self.assertEqual(tuple(f([c]) for f in factories), ([10], [90]))


                         
                         

                          
# Tests for the configuration/command line parsing functions
                          

def parse_cl_test(opts, args, out, debug=False):
    def t(self):
        self.assertEqual(parse_command_line(opts, args.split()), out)
    return t

def parse_cl_test_fail(opts, args, exc=Exception):
    def t(self):
        self.assertRaises(exc,
                          lambda: parse_command_line(opts, args.split()))
    return t
        
class t_parse_command_line(unittest.TestCase):
    opts_1 = (Option("foo"), Option("bar"))

    # Basic functionality
    test_multiple_1 = parse_cl_test(opts_1,
                                    "--foo=1 --bar=2",
                                    ({'foo': '1', 'bar': '2'}, tuple()))
    
    test_multiple_2 = parse_cl_test(opts_1,
                                    "--foo 1 --bar 2",
                                    ({'foo': '1', 'bar': '2'}, tuple()))
    
    # Positional arguments are returned untouched
    test_positional_1 = parse_cl_test(opts_1,
                                      "--foo 1 --bar 2 some more args",
                                      ({'foo': '1', 'bar': '2'},
                                       tuple("some more args".split())))
    
    # Value names are respected
    opts_2 = (Option("foo", valname="foo2"),)
    test_valname_1 = parse_cl_test(opts_2,
                                   "--foo 1",
                                   ({'foo2': '1'}, tuple()))


    # Default values are not applied
    opts_3 = (Option("foo", default_val="12"),
              Option("bar"))
    test_defaults_1 = parse_cl_test(opts_3,
                                    "--bar 2",
                                    ({'bar': '2'}, tuple()))

    # Missing values are ignored
    test_missing_1 = parse_cl_test(opts_3,
                                   "--foo 1",
                                   ({'foo': '1'}, tuple()))

    # Unexpected values, however, generate an error
    test_unexpected = parse_cl_test_fail(opts_3,
                                         "--quux 12",
                                         exc=ToolConfigurationError)

    # Conversions are not done
    opts_4 = (Option("foo", converter=int),)
    test_conversion_1 = parse_cl_test(opts_4,
                                      "--foo 10",
                                      ({'foo': "10"}, tuple()))

    # No options supplied
    test_no_options_1 = parse_cl_test(tuple(),
                                      "this can be anything",
                                      ({},
                                       tuple("this can be anything".split())))

    # Flags are handled properly
    test_flags_1 = parse_cl_test((Flag("foo"),),
                                 "--foo",
                                 ({'foo': True}, tuple()))


    # -- FalseFlags are handled like normal flags! (We are just
    #    signaling that the user specified the flag, not what its
    #    ultimate truth value is.
    test_flags_2 = parse_cl_test((FalseFlag("foo"),),
                                 "--foo",
                                 ({'foo': True}, tuple()))

    
    # --help works
    def test_help_1(self):
        # Hijack stdout and stderr temporarily so we can catch what
        # optparse outputs.
        serr = sys.stderr = StringIO()
        sout = sys.stdout = StringIO()
        try:
            opts = (Option("foo"), Option("bar", hlp="some help"))
            try:
                out = parse_command_line(opts, "--help".split())
            except SystemExit, e:
                self.assertEqual(e.code, 0)
        finally:
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__

        def dedent(bigstring):
            sio = StringIO(bigstring)
            return "".join(line.lstrip()[1:] for line in sio)

        self.assertEqual(
            sout.getvalue(),
            dedent("""|Usage: run_unit_tests.py [options] - 
                      |
                      |Options:
                      |  -h, --help  show this help message and exit
                      |  --bar=ARG   some help
                      |"""))


    



import ConfigParser
def cparser_from_dict(d):
    cp = ConfigParser.ConfigParser()
    for sct, opts in d.items():
        if sct != 'DEFAULT':
            cp.add_section(sct)
        for k, v in opts.items():
            cp.set(sct, k, v)
    return cp

def parse_cfg_test(opts, cmd_name, cfg_dict, out, no_sections=False):
    cp = cparser_from_dict(cfg_dict)
    def t(self):
        self.assertEqual(
            parse_config_file(opts, cp, cmd_name, no_sections=no_sections),
            out)
    return t

def parse_cfg_test_fail(
    opts, cmd_name, cfg_dict, exc=Exception, no_sections=False):
    cp = cparser_from_dict(cfg_dict)
    def t(self):
        self.assertRaises(
            exc,
            lambda: parse_config_file(
                opts, cp, cmd_name, no_sections=no_sections))
    return t
    
class t_parse_config_file(unittest.TestCase):
    # Basic functionality
    opts_1 = (Option("foo"), Option("bar"))
    test_basic_1 = parse_cfg_test(opts_1,
                                  'ryfoo',
                                  {'ryfoo' : { 'foo': '1',
                                               'bar': 'baz' }},
                                  {'foo': '1', 'bar': 'baz'})

    # Both ALL and <cmdname> sections are used
    test_sections_1 = parse_cfg_test(opts_1,
                                     'ryfoo',
                                     {'ALL' : {'foo': '1', 'bar': 'baz'}},
                                     {'foo': '1', 'bar': 'baz'})

    test_sections_2 = parse_cfg_test(opts_1,
                                     'ryfoo',
                                     {'ALL'  : {'foo': '1'},
                                      'ryfoo': {'bar': 'baz'}},
                                     {'foo': '1', 'bar': 'baz'})
    
    # If something is in ALL and <cmdname>, <cmdname> wins
    test_sections_3 = parse_cfg_test(opts_1,
                                     'ryfoo',
                                     {'ALL': {'foo': '1'},
                                      'ryfoo': {'foo': '2', 'bar': 'baz'}},
                                     {'foo': '2', 'bar': 'baz'})

    # If no_sections is True, just use the DEFAULT section for all our
    # configuration (TODO: On reflection, this is probably a
    # misfeature. Remove.)
    test_sections_4 = parse_cfg_test(opts_1,
                                     '',
                                     {'DEFAULT': {'foo': '1', 'bar': 'baz'}},
                                     {'foo': '1', 'bar': 'baz'},
                                     no_sections=True)

    # test_sections_5 = parse_cfg_test(opts_1,
    #                                  'ryfoo',
    #                                  {'DEFAULT': {'foo': '1', 'bar': 'baz'},
    #                                   'ryfoo': {'foo': 'wrong', 'bar': 'XXX'},
    #                                  {'foo': '1', 'bar': 'baz'},
    #                                  no_sections=True)

    # If no_sections is True, we ignore the <cmd_name> and ALL
    # sections, too.
    test_sections_6 = parse_cfg_test(opts_1,
                                     '',
                                     {'DEFAULT': {'foo': '1', 'bar': 'baz'},
                                      'ALL': {'foo': '2'},
                                      'ryfoo': {'foo': '3'}},
                                     {'foo': '1', 'bar': 'baz'},
                                     no_sections=True)


    # Additional/irrelevant sections are ignored
    test_sections_7 = parse_cfg_test(opts_1,
                                     'ryfoo',
                                     {'ALL': {'foo': '1'},
                                      'ryfoo': {'foo': '2', 'bar': 'baz'},
                                      'rybar': {'foo': '1234'}},
                                     {'foo': '2', 'bar': 'baz'})
    
    test_sections_8 = parse_cfg_test(opts_1,
                                     'ryfoo',
                                     {'DEFAULT': {'foo': '1', 'bar': 'baz'},
                                      'rybar': {'foo': '2'}},
                                     {'foo': '1', 'bar': 'baz'},
                                     no_sections=True)

    # Default values are not applied
    opts_2 = (Option("foo", default_val="12"), Option("bar"))
    test_defaults_1 = parse_cfg_test(opts_2,
                                     'ryfoo',
                                     {'ryfoo': {'bar': '1'}},
                                     {'bar': '1'})
        

    # Missing values are ignored
    test_missing_1 = parse_cfg_test(opts_1,
                                    'ryfoo',
                                    {'ryfoo': {'bar': '1'}},
                                    {'bar': '1'})


    # Missing sections are ignored (return an empty dictionary instead)
    test_empty_sections_1 = parse_cfg_test(opts_1, '', {}, {})
    
    test_empty_sections_2 = parse_cfg_test(opts_1, 'ryfoo', {}, {})

    test_empty_sections_3 = parse_cfg_test(opts_1, '', {}, {}, no_sections=True)



    # However, unexpected keys in an expected section raise an
    # exception....
    test_unexpected_keys_1 = parse_cfg_test_fail(opts_1,
                                                 'ryfoo',
                                                 {'ryfoo' : {'quux': "10"}})

    # ...Unless that section is [ALL], in which case they're just
    # ignored.
    test_unexpected_keys_2 = parse_cfg_test(opts_1,
                                            'ryfoo',
                                            {'ALL'  : {'quux': "10"},
                                             'ryfoo': {'foo': '1', 'bar': '2'}},
                                            {'foo': '1', 'bar': '2'})
                                            

    # Conversions are not done
    opts_2 = (Option("foo", converter=int),)
    test_convert_1 = parse_cfg_test(opts_2,
                                    'ryfoo',
                                    {'ryfoo': {'foo': '1',}},
                                    {'foo' : "1"})

    # Assignment to custom valnames happens
    opts_3 = (Option("foo", valname="foo2"),
              Option("bar", valname="bar2"))
    test_valname_1 = parse_cfg_test(opts_3,
                                    'ryfoo',
                                    {'ALL': {'foo': "12"},
                                     'ryfoo': {'bar': "baz"}},
                                    {'foo2': "12", 'bar2': "baz"})

                                           
    

def test_merge_configuration(opts, configs, out):
    def t(self):
        self.assertEquals(out, merge_configuration(opts, *configs))
    return t

def test_merge_configuration_fails(opts, configs, exc=Exception):
    def t(self):
        self.assertRaises(exc, lambda: merge_configuration(opts, *configs))
    return t


class t_merge_configuration(unittest.TestCase):
    opts_1 = (Option("foo"), Option("bar"), Option("baz"))

    # Values get merged
    test_merge_1 = test_merge_configuration(
        opts_1,
        ({'foo': 'a'}, {'bar': 'b'}, {'baz': 'c'}),
        {'foo': 'a', 'bar': 'b', 'baz': 'c'})

    # Values defined in subsequent dicts overwrite values in previous
    # dicts
    test_overwrite_1 = test_merge_configuration(
        opts_1,
        ({'foo': 'a', 'bar': 'b'}, {'foo': 'c', 'baz': 'd'}),
        {'foo': 'c', 'bar': 'b', 'baz': 'd'})

    # If 'ignore_config' is set in any dict, previous configurations
    # are ignored (i.e., the configuration environment is reset
    # starting with that dict)
    test_ignore_config_1 = test_merge_configuration(
        (Option("foo", default_val="albert"),
         Option("bar", default_val="hoffman")),
        ({'foo': 'dustin', 'bar': 'pedroia'},
         {'ignore_config': 'true', 'foo': "abby"}),
        {'foo': "abby", 'bar': "hoffman"})
         


    # Defaults ARE applied
    test_defaults_1 = test_merge_configuration(
        (Option("foo", default_val="philip"),
         Option("bar"),
         Option("baz")),
        ({'bar': 'seymour'}, {'baz': 'hoffman'}),
        {'foo': 'philip', 'bar': 'seymour', 'baz': 'hoffman'})

    # Default values are assigned to value name
    test_defaults_2 = test_merge_configuration(
        (Option("foo", valname="fu", default_val="philip"),
         Option("bar"),
         Option("baz")),
        ({'bar': 'seymour'}, {'baz': 'hoffman'}),
        {'fu': 'philip', 'bar': 'seymour', 'baz': 'hoffman'})
        

    # Missing default values are returned with NO_DEFAULT. Calling
    # code (e.g., rytools.execute) should check for NO_DEFAULT values
    # and error out if appropriate.
    test_missing_values_1 = test_merge_configuration(
        (Option('foo'), Option('bar')),
        ({'bar': 'c'},),
        {'foo': NO_DEFAULT, 'bar': 'c'})

    # Conversions ARE done
    test_conversion_1 = test_merge_configuration(
        (Option("foo", converter=int), Option("bar")),
        ({'foo': "1", 'bar': "c"},),
        {'foo': 1, 'bar': 'c'})

    # Flags and FalseFlags are handled properly
    test_merge_flags_1 = test_merge_configuration(
        (Flag("foo"), FalseFlag("no-bar", valname='bar')),
        ({'foo': True, 'bar': True},),
        {'foo': True, 'bar': False})


import tempfile
class TempConfigFile(object):
    def __init__(self, cfg_dict):
        self.cparser = cparser_from_dict(cfg_dict)
        self.cfile = tempfile.NamedTemporaryFile()
        cparser.write(self.cfile)
    def get_name(self):
        return self.cfile.name
        
                
       



    
if __name__ == '__main__':
    unittest.main()
